import tablate as tb
from tablate import HtmlContainerStylesInput

tab = tb.Tablate(html_container_styles=HtmlContainerStylesInput(html_container_border_style="thin",
                                                                html_container_padding=20))

table_columns = [
    {
        "key": "col_one",
        "string": "Col One",
        "width": "50%"
    },
    {
        "key": "col_two",
        "string": "Col One",
    },
    {
        "key": "col_three",
        "string": "Col One",
    }
]

table_rows = [
    {
        "col_one": "Column One, Row One",
        "col_two": "Column Two, Row One",
        "col_three": "Column Three, Row One"
    },
    {
        "col_one": "Column One, Row Two",
        "col_two": "Column Two, Row Two",
        "col_three": "Column Three, Row Two"
    },
    {
        "col_one": "Column One, Row Three",
        "col_two": "Column Two, Row Three",
        "col_three": "Column Three, Row Three"
    }
]

tab = tb.Tablate()

tab.add_text_frame("This is an basic example for Tablate usage.")
tab.add_grid_frame(["One", "Two", "Three"])
tab.add_table_frame(columns=table_columns, rows=table_rows)
tab.print()
# or
print(tab.to_html())
tab.list_frames()

some_other_one = tb.Tablate()
some_other_one.add_text_frame("lalala")
some_other_one.add_text_frame("lalala")
some_other_one.add_text_frame("lalala")


tab.replace_frame(1, some_other_one)
tab.list_frames()

print(tab)


from tablate.library.initializers.mappers.element.base.base_column_mapper import base_column_mapper
from tablate.library.initializers.mappers.element.base.base_frame_mapper import base_frame_mapper
from tablate.library.initializers.mappers.element.base.base_text_mapper import base_text_mapper
from tablate.library.initializers.mappers.element.html.html_column_mapper import html_column_mapper
from tablate.library.initializers.mappers.element.html.html_frame_mapper import html_frame_mapper
from tablate.library.initializers.mappers.element.html.html_outer_mapper import html_outer_mapper
from tablate.library.initializers.mappers.element.html.html_text_mapper import html_text_mapper
from tablate.library.initializers.mappers.object.element.html_column_input_to_style import html_column_input_to_style
from tablate.library.initializers.mappers.object.element.html_frame_input_to_style import html_frame_input_to_style
from tablate.library.initializers.mappers.object.element.html_outer_input_style import html_outer_input_to_style
from tablate.library.initializers.mappers.object.element.html_text_input_to_style import html_text_input_to_style
from tablate.type.defaults import container_border_default, container_padding_default, container_width_default, \
    background_padding_default, html_px_multiplier_default
from tablate.type.primitives import ContainerBorder, ContainerPadding, ContainerWidth, FrameDivider, Background, BackgroundPadding, \
    HtmlPxMultiplier, HtmlDefaultColors
from tablate.type.type_style import FrameStyles, OuterStyles
from tablate.type.type_global import ConsoleGlobals, HtmlGlobals, Globals
from tablate.type.type_input import HtmlContainerStylesInput, ColumnStylesInput, TextStylesInput, HtmlFrameStylesInput, \
    HtmlColumnStylesInput, HtmlTextStylesInput


def globals_init(container_border: ContainerBorder = None,
                 container_padding: ContainerPadding = None,
                 container_width: ContainerWidth = None,

                 html_default_colors: HtmlDefaultColors = None,

                 frame_divider: FrameDivider = None,
                 background: Background = None,
                 background_padding: BackgroundPadding = None,

                 html_px_multiplier: HtmlPxMultiplier = None,
                 html_container_styles: HtmlContainerStylesInput = None,

                 column_styles: ColumnStylesInput = None,
                 text_styles: TextStylesInput = None,

                 html_frame_styles: HtmlFrameStylesInput = None,

                 html_column_styles: HtmlColumnStylesInput = None,
                 html_text_styles: HtmlTextStylesInput = None) -> Globals:

    html_container_styles = html_outer_input_to_style(html_container_styles)
    html_frame_styles = html_frame_input_to_style(html_frame_styles)
    html_column_styles = html_column_input_to_style(html_column_styles)
    html_text_styles = html_text_input_to_style(html_text_styles)

    default_outer_styles = OuterStyles(container_border=container_border if container_border else container_border_default,
                                       container_padding=container_padding if container_padding else container_padding_default,
                                       container_width=container_width if container_width else container_width_default,
                                       background_padding=background_padding if background_padding else background_padding_default)

    default_frame_styles = base_frame_mapper(frame_input=FrameStyles(frame_divider=frame_divider,
                                                                     background=background))

    default_column_styles = base_column_mapper(columns_input=column_styles)

    default_text_styles = base_text_mapper(text_input=text_styles)

    console_globals = ConsoleGlobals(outer_styles=default_outer_styles,
                                     frame_styles=default_frame_styles,
                                     column_styles=default_column_styles,
                                     text_styles=default_text_styles)

    if html_px_multiplier is None:
        html_px_multiplier = html_px_multiplier_default

    default_html_outer = html_outer_mapper(html_outer_input=html_container_styles,
                                           base_outer_defaults=default_outer_styles,
                                           html_px_multiplier=html_px_multiplier,
                                           html_default_colors=html_default_colors)
    default_html_frame = html_frame_mapper(html_frame_input=html_frame_styles,
                                           base_frame_defaults=default_frame_styles)
    default_html_column = html_column_mapper(html_columns_input=html_column_styles,
                                             base_column_defaults=default_column_styles)
    default_html_text = html_text_mapper(html_text_input=html_text_styles,
                                         base_text_defaults=default_text_styles)

    html_globals = HtmlGlobals(html_container_styles=default_html_outer,
                               html_frame_styles=default_html_frame,
                               html_column_styles=default_html_column,
                               html_text_styles=default_html_text,
                               css_injection="",
                               column_baselines=[],
                               styler=None)

    return Globals(console=console_globals, html=html_globals)



from copy import deepcopy
from typing import List, Union

from tablate.classes.bases.TablateApiSet import TablateApiSet
from tablate.library.calcs.gen_frame_name import gen_frame_name
from tablate.library.initializers.processors.process_frame import process_frame
from tablate.library.initializers.table_init import table_init
from tablate.type.primitives import FrameDivider, Multiline, TextColor, \
    TextAlign, TextStyle, MaxLines, HideHeader, Background, ColumnDivider, ColumnPadding, HtmlPxMultiplier, \
    BackgroundPadding, FrameName, ContainerBorder, ContainerPadding, ContainerWidth, HtmlDefaultColors
from tablate.type.type_input import GridColumnInput, TableColumnInput, TableRowsInput, HtmlContainerStylesInput, \
    ColumnStylesInput, TextStylesInput, HtmlFrameStylesInput, \
    HtmlColumnStylesInput, HtmlTextStylesInput, HtmlTextFrameStylesInput, HtmlGridFrameStylesInput, \
    HtmlTableHeaderStylesInput, HtmlTableFrameStylesInput, \
    TableHeaderFrameStylesInput
from tablate.type.type_store import FrameDict


class Tablate(TablateApiSet):
    """
    Container for text, grid and table frames. Allows cumulative addition of frames to an instance.

     Note:
         When using the 'Tablate' class in an IPython environment, it is important to name each frame (or else rerunning cells will create duplicate entries).

    Args:
        container_border: outer border style for the Tablate container instance.
            [default: 'thick']
        container_padding: outer padding (in Unicode character width) for the Tablate container instance.
            [default: 1]
        container_width: outer width (in Unicode character width) for the Tablate container instance.
            [default: 120 or terminal width]
        html_default_colors: apply the default HTML color scheme to the Tablate instance.
            [default: True]
        frame_divider: default dividing line between frames within the Tablate container instance.
            [default: 'thick']
        background: default background for frames withing the Tablate container instance.
            [default: None]
        background_padding: additional padding applied to cells with a background color set (ASCII only).
            [default: 1]
        html_px_multiplier: Multiplier applied to HTML px properties.
            [default: 1]
        html_container_styles: style dictionary for HTML specific styles for Tablate container instance.
            [default: None]
        column_styles: style dictionary for default base column styles.
            [default: {"column_divider": 'thin', "padding": 1, "background_padding": 1}]
        text_styles: style dictionary for default base text styles.
            [default: {
            "text_style": 'normal',
            "text_align": 'left',
            "text_color": None
            }]
        html_frame_styles: style dictionary for default HTML specific frame styles.
            [default: {\n
            "html_frame_divider_style": "thick",
            "html_frame_divider_weight: 1,
            "html_frame_divider_color": 'black',
            "html_multiline": True (False for Table frames),
            "html_max_lines": None,
            "html_background": None
            }]
        html_column_styles: style dictionary for default HTML specific column styles.
            [default: {
            "html_column_divider_style": 'thin',
            "html_column_divider_weight": 1,
            "html_column_divider_color": 'black',
            "html_padding": 6px
            }]
        html_text_styles: style dictionary for default HTML specific text styles.
            [default: {
            "html_text_style": 'normal',
            "html_text_align": 'left',
            "html_text_color": None,
            "html_text_size": 16
            }]
    """

    def __init__(self,
                 container_border: ContainerBorder = None,
                 container_padding: ContainerPadding = None,
                 container_width: ContainerWidth = None,

                 html_default_colors: HtmlDefaultColors = None,

                 frame_divider: FrameDivider = None,
                 background: Background = None,
                 background_padding: BackgroundPadding = None,

                 html_px_multiplier: HtmlPxMultiplier = None,
                 html_container_styles: HtmlContainerStylesInput = None,

                 column_styles: ColumnStylesInput = None,
                 text_styles: TextStylesInput = None,

                 html_frame_styles: HtmlFrameStylesInput = None,

                 html_column_styles: HtmlColumnStylesInput = None,
                 html_text_styles: HtmlTextStylesInput = None) -> None:

        args = deepcopy(locals())
        del args["self"]

        TablateApiSet.__init__(self=self, **args)

    def add_text_frame(self,
                       text: Union[str, int, float],

                       name: FrameName = None,

                       text_style: TextStyle = None,
                       text_align: TextAlign = None,
                       text_color: TextColor = None,

                       frame_divider: FrameDivider = None,
                       frame_padding: ColumnPadding = None,
                       background: Background = None,
                       background_padding: BackgroundPadding = None,
                       multiline: Multiline = None,
                       max_lines: MaxLines = None,

                       html_px_multiplier: HtmlPxMultiplier = None,
                       html_styles: HtmlTextFrameStylesInput = None) -> None:
        """
        Add a new text frame (single row, single column) to Tablate container instance.

        Args:
            text: Text frame content.
                [required]
            name: Frame name (to prevent duplicate entries).
                [default: None]
            text_style: Text style for text frame.
                [default: 'normal']
            text_align: Text alignment for text frame.
                [default: 'left']
            text_color: Text color for text frame.
                [default: 'None']
            frame_divider: Dividing line at the bottom of the frame.
                [default: 'thick']
            frame_padding: Text padding within text frame.
                [default: 1]
            background: Background color for the text frame.
                [default: None]
            background_padding: Additional padding applied to text if background color set (ASCII only).
                [default: 1]
            multiline: Whether the text may wrap to multiple lines.
                [default: True]
            max_lines: Maximum number of lines to which the text may wrap.
                [default: None]
            html_px_multiplier: Multiplier applied to HTML px properties.
                [default: 1]
            html_styles: HTML specific styles for text frame.
                [default: {
                "html_frame_styles": {"html_frame_divider_style": "thick", "html_frame_divider_weight: 1, "html_frame_divider_color": 'black', "html_multiline": True (False for Table frames), "html_max_lines": None, "html_background": None },
                "html_text_styles": {"html_text_style": 'normal', "html_text_align": 'left', "html_text_color": None, "html_text_size": 16 }
                }]
        """
        frame_dict = process_frame(frame_name=name,
                                   frame_type="text",
                                   frame_args=locals(),
                                   frame_list=self._frame_list,
                                   global_options=self._globals_store.store)

        self._frame_list[frame_dict.name] = frame_dict

    def add_grid_frame(self,
                       columns: List[Union[str, GridColumnInput]],

                       name: FrameName = None,

                       frame_divider: FrameDivider = None,
                       background: Background = None,
                       background_padding: BackgroundPadding = None,
                       multiline: Multiline = None,
                       max_lines: MaxLines = None,

                       column_divider: ColumnDivider = None,
                       column_padding: ColumnPadding = None,

                       text_style: TextStyle = None,
                       text_align: TextAlign = None,
                       text_color: TextColor = None,

                       html_px_multiplier: HtmlPxMultiplier = None,
                       html_styles: HtmlGridFrameStylesInput = None) -> None:
        """

        Add a new grid frame (single row of multiple columns) to Tablate instance.

        Args:
            columns: Either a list of contents (str, int, float) for columns or a list of column dicts containing contents and styles for columns.
                [required]
            name: Frame name (to prevent duplicate entries).
                [default: None]
            frame_divider: Dividing line at the bottom of the frame.
                [default: 'thick']
            background: Background color for the grig frame.
                [default: None]
            background_padding: Additional padding applied to text if background color set (ASCII only).
                [default: 1]
            multiline: Whether the text may wrap to multiple lines.
                [default: True]
            max_lines: Maximum number of lines to which the text may wrap.
                [default: None]
            column_divider: Dividing line between columns.
                [default: 'thin']
            column_padding: Text padding within each column.
                [default: 1]
            text_style: Text style for grid frame.
                [default: 'normal']
            text_align: Text alignment for grid frame.
                [default: 'left']
            text_color: Text color for grid frame.
                [default: 'None']
            html_px_multiplier: Multiplier applied to HTML px properties.
                [default: 1]
            html_styles: HTML specific styles for grid frame.
                [default: {
                "html_frame_styles": {"html_frame_divider_style": "thick", "html_frame_divider_weight: 1, "html_frame_divider_color": 'black', "html_multiline": True (False for Table frames), "html_max_lines": None, "html_background": None },
                "html_column_styles": {"html_column_divider_style": "thin", "html_column_divider_weight": 1, "html_column_divider_color": "black", "html_padding": "6px"}
                "html_text_styles": {"html_text_style": 'normal', "html_text_align": 'left', "html_text_color": None, "html_text_size": 16 }
                }]
        """
        frame_dict = process_frame(frame_name=name,
                                   frame_type="grid",
                                   frame_args=locals(),
                                   frame_list=self._frame_list,
                                   global_options=self._globals_store.store)

        self._frame_list[frame_dict.name] = frame_dict

    def add_table_frame(self,
                        columns: List[TableColumnInput],
                        rows: List[TableRowsInput],

                        name: FrameName = None,

                        frame_divider: FrameDivider = None,
                        background: Background = None,
                        background_padding: BackgroundPadding = None,
                        multiline: Multiline = None,
                        max_lines: MaxLines = None,

                        multiline_header: Multiline = None,
                        max_lines_header: MaxLines = None,
                        hide_header: HideHeader = None,

                        column_divider: ColumnDivider = None,
                        column_padding: ColumnPadding = None,
                        header_base_divider: FrameDivider = None,

                        row_line_divider: FrameDivider = None,
                        odd_row_background: Background = None,
                        even_row_background: Background = None,

                        text_style: TextStyle = None,
                        text_align: TextAlign = None,
                        text_color: TextColor = None,

                        header_styles: TableHeaderFrameStylesInput = None,

                        html_px_multiplier: HtmlPxMultiplier = None,
                        html_styles: HtmlTableFrameStylesInput = None,

                        html_header_styles: HtmlTableHeaderStylesInput = None) -> None:
        """
        Add a new table frame (multiple rows of multiple columns) to Tablate instance.

        Args:
            columns: List of column dicts containing a required column key and optional (str, int, float) to display, as well as column specific styling options.
                [required]
            rows: List of row dicts containing the (str, int, float) values for each column key.
                [required]
            name: Frame name (to prevent duplicate entries).
                [default: None]
            frame_divider: Dividing line at the bottom of the frame.
                [default: 'thick']
            background: Background color for the table frame.
                [default: None]
            background_padding: Additional padding applied to text if background color set (ASCII only).
                [default: 1]
            multiline: Whether the text may wrap to multiple lines.
                [default: False]
            max_lines: Maximum number of lines to which the text may wrap.
                [default: None]
            multiline_header: Whether the header text may wrap to multiple lines.
                [default: False]
            max_lines_header: Maximum number of lines to which the header text may wrap.
                [default: None]
            hide_header: Whether to not render the header.
                [default: False]
            column_divider: Dividing line between columns.
                [default: 'thin']
            column_padding: Text padding within each column.
                [default: 1]
            header_base_divider: Dividing line at the bottom of the table header frame.
                [default: 'thick']
            row_line_divider: Dividing line between each row of the table frame.
                [default: 'thin']
            odd_row_background: Background color for odd rows of table frame.
                [default: None]
            even_row_background: Background color for even rows of table frame.
                [default: None]
            text_style: Text style for table frame.
                [default: 'normal']
            text_align: Text alignment for table frame.
                [default: 'left']
            text_color: Text color for grid frame.
                [default: None]
            header_styles: Specific styles to apply to the table header frame.
                [default: None (inherits table styles)]
            html_px_multiplier: Multiplier applied to HTML px properties.
                [default: 1]
            html_styles: HTML specific styles for table frame.
                [default: {
                "html_frame_styles": {"html_frame_divider_style": "thick", "html_frame_divider_weight: 1, "html_frame_divider_color": 'black', "html_multiline": True (False for Table frames), "html_max_lines": None, "html_background": None },
                "html_column_styles": {"html_column_divider_style": "thin", "html_column_divider_weight": 1, "html_column_divider_color": "black", "html_padding": "6px"},
                "html_text_styles": {"html_text_style": 'normal', "html_text_align": 'left', "html_text_color": None, "html_text_size": 16 },
                "html_row_styles": {"html_row_line_divider_weight": 1, "html_row_line_divider_style": "thin", "html_row_line_divider_color": None, "html_odds_background": None, "html_evens_background": None}
                }] (Tablate constructor option `html_default_colors` applies styles to table headers and even rows, separately from, but overwritten by, this html_styles dict)
            html_header_styles: HTML specific styles for table header frame.
                [default: None (inherits table styles)]
        """
        frame_dict = process_frame(frame_name=name,
                                   frame_type="table",
                                   frame_args=locals(),
                                   frame_list=self._frame_list,
                                   global_options=self._globals_store.store)

        self._frame_list[frame_dict.name] = frame_dict

    def from_dict(self,
                  dict_object: dict,
                  name: FrameName = None,
                  capitalize_keys: bool = True,
                  frame_divider: FrameDivider = None,
                  multiline: Multiline = None,
                  max_lines: MaxLines = None,
                  background: Background = None,
                  background_padding: BackgroundPadding = None,

                  multiline_header: Multiline = None,
                  max_lines_header: MaxLines = None,
                  hide_header: HideHeader = None,

                  column_divider: ColumnDivider = None,
                  column_padding: ColumnPadding = None,
                  header_base_divider: FrameDivider = None,

                  row_line_divider: FrameDivider = None,
                  odd_row_background: Background = None,
                  even_row_background: Background = None,

                  text_style: TextStyle = None,
                  text_align: TextAlign = None,
                  text_color: TextColor = None,

                  header_styles: TableHeaderFrameStylesInput = None,

                  html_px_multiplier: HtmlPxMultiplier = None,
                  html_styles: HtmlTableFrameStylesInput = None,

                  html_header_styles: HtmlTableHeaderStylesInput = None):
        """
        Creates a table frame from a dict in three possible formats:
            - {"one": [1, 2, 3], "two": [4, 5, 6]}
            - {'one': {0: 1, 1: 2, 2: 3}, 'two': {0: 1, 1: 2, 2: 3}}
            - {'one': {(1, 'red'): 1, (1, 'blue'): 2, (2, 'green'): 3}, 'two': {(1, 'red'): 1, (1, 'blue'): 2, (2, 'green'): 3}

        Pandas returns the latter two dict types from its `.to_dict()` method.

        Args:
            dict_object: Input dict.
                [required]
            name: Frame name (to prevent duplicate entries).
                [default: None]
            capitalize_keys: Whether to capitalize Pandas column keys.
                [default: True]
            frame_divider: Dividing line at the bottom of the frame.
                [default: 'thick']
            background: Background color for the table frame.
                [default: None]
            background_padding: Additional padding applied to text if background color set (ASCII only).
                [default: 1]
            multiline: Whether the text may wrap to multiple lines.
                [default: False]
            max_lines: Maximum number of lines to which the text may wrap.
                [default: None]
            multiline_header: Whether the header text may wrap to multiple lines.
                [default: False]
            max_lines_header: Maximum number of lines to which the header text may wrap.
                [default: None]
            hide_header: Whether to not render the header.
                [default: False]
            column_divider: Dividing line between columns.
                [default: 'thin']
            column_padding: Text padding within each column.
                [default: 1]
            header_base_divider: Dividing line at the bottom of the table header frame.
                [default: 'thick']
            row_line_divider: Dividing line between each row of the table frame.
                [default: 'thin']
            odd_row_background: Background color for odd rows of table frame.
                [default: None]
            even_row_background: Background color for even rows of table frame.
                [default: None]
            text_style: Text style for table frame.
                [default: 'normal']
            text_align: Text alignment for table frame.
                [default: 'left']
            text_color: Text color for grid frame.
                [default: None]
            header_styles: Specific styles to apply to the table header frame.
                [default: None (inherits table styles)]
            html_px_multiplier: Multiplier applied to HTML px properties.
                [default: 1]
            html_styles: HTML specific styles for table frame.
                [default: {
                "html_frame_styles": {"html_frame_divider_style": "thick", "html_frame_divider_weight: 1, "html_frame_divider_color": 'black', "html_multiline": True (False for Table frames), "html_max_lines": None, "html_background": None },
                "html_column_styles": {"html_column_divider_style": "thin", "html_column_divider_weight": 1, "html_column_divider_color": "black", "html_padding": "6px"},
                "html_text_styles": {"html_text_style": 'normal', "html_text_align": 'left', "html_text_color": None, "html_text_size": 16 },
                "html_row_styles": {"html_row_line_divider_weight": 1, "html_row_line_divider_style": "thin", "html_row_line_divider_color": None, "html_odds_background": None, "html_evens_background": None}
                }] (Tablate constructor option `html_default_colors` applies styles to table headers and even rows, separately from, but overwritten by, this html_styles dict)
            html_header_styles: HTML specific styles for table header frame.
                [default: None (inherits table styles)]
        """
        name = gen_frame_name(name=name, type="table", frame_dict=self._frame_list)

        args = deepcopy(locals())
        del args["self"]
        del args["dict_object"]
        del args["capitalize_keys"]

        columns = []
        rows = []
        for col_index, (col_key, col_value) in enumerate(dict_object.items()):
            if type(col_value) == dict:
                for row_index, (row_value_index, row_value) in enumerate(col_value.items()):
                    if col_index == 0:
                        rows.append({})
                        if type(row_value_index) == tuple:
                            for i in range(0, len(row_value_index)):
                                if row_index == 0:
                                    columns.append({"key": f"i{i}", "width": "10%", "text_align": "right"})
                                rows[row_index][f"i{i}"] = row_value_index[i]
                        else:
                            if row_index == 0:
                                columns.append({"key": "i", "width": "10%", "text_align": "right"})
                            rows[row_index]["i"] = row_value_index
                    rows[row_index][col_key.title() if capitalize_keys else col_key] = row_value
            elif type(col_value) == list:
                for row_index, row_value in enumerate(col_value):
                    if col_index == 0:
                        rows.append({})
                    rows[row_index][col_key.title() if capitalize_keys else col_key] = row_value
            columns.append({"key": col_key.title() if capitalize_keys else col_key})

        # new_table = Table(columns=columns, rows=rows, **args)
        # todo: fix column widths... ensure no bugs if millions of index columns
        # todo: for later allow specific column widths
        table_store = table_init(**args,
                                 columns=columns,
                                 rows=rows,
                                 global_options=self._globals_store.store)
        self._frame_list[name] = FrameDict(name=name,
                                           type="table",
                                           args={"columns": columns, "rows": rows, **args},
                                           store=table_store)



from typing import TypedDict, Dict, Optional

from tablate.type.primitives import TextString, FrameDivider, ColumnWidth, TextStyle, TextAlign, Background, \
    ColumnPadding, TextColor, ColumnDivider, HtmlDividerWeight, HtmlColumnDividerStyle, HtmlColumnPadding, \
    HtmlTextStyle, HtmlTextAlign, HtmlTextColor, HtmlBackground, BackgroundPadding, HtmlTextSize, MaxLines, \
    Multiline, HtmlFrameDivider, HtmlContainerBorder, HtmlContainerPadding, HtmlContainerWidth, HtmlDividerColor


class HtmlContainerStylesInput(TypedDict, total=False):  # HtmlContainerBase
    html_container_border_weight: HtmlDividerWeight
    html_container_border_style: HtmlContainerBorder
    html_container_border_color: HtmlDividerColor
    html_container_padding: HtmlContainerPadding
    html_container_width: HtmlContainerWidth


########################################################################################################################

class FrameStylesInput(TypedDict, total=False):  # FrameBase
    frame_divider: FrameDivider
    max_lines: MaxLines
    multiline: Multiline
    background: Background


class HtmlFrameStylesInput(TypedDict, total=False):  # HtmlFrameBase
    html_frame_divider_style: HtmlFrameDivider
    html_frame_divider_weight: HtmlDividerWeight
    html_frame_divider_color: HtmlDividerColor
    html_max_lines: MaxLines
    html_multiline: Multiline
    html_background: HtmlBackground


########################################################################################################################

class ColumnStylesInput(TypedDict, total=False):  # ColumnBase
    column_divider: ColumnDivider
    padding: ColumnPadding
    background_padding: BackgroundPadding


class TextStylesInput(TypedDict, total=False):  # TextBase
    text_style: TextStyle
    text_align: TextAlign
    text_color: TextColor


class HtmlColumnStylesInput(TypedDict, total=False):  # HtmlColumnBase
    html_column_divider_style: HtmlColumnDividerStyle
    html_column_divider_weight: HtmlDividerWeight
    html_column_divider_color: HtmlDividerColor
    html_padding: HtmlColumnPadding


class HtmlTextStylesInput(TypedDict, total=False):  # HtmlTextBase
    html_text_style: HtmlTextStyle
    html_text_align: HtmlTextAlign
    html_text_color: HtmlTextColor
    html_text_size: HtmlTextSize

    
class RowsStylesInput(TypedDict, total=False):  # TableRowsBase
    row_line_divider: FrameDivider
    odds_background: Background
    evens_background: Background

    
class HtmlRowsStylesInput(TypedDict, total=False):  # HtmlTableRowsBase
    html_row_line_divider_weight: HtmlDividerWeight
    html_row_line_divider_style: HtmlFrameDivider
    html_row_line_divider_color: HtmlDividerColor
    html_odds_background: HtmlBackground
    html_evens_background: HtmlBackground


########################################################################################################################
########################################################################################################################
########################################################################################################################

class HtmlColumnInput(TypedDict, total=False):
    padding: HtmlColumnPadding
    divider_style: HtmlColumnDividerStyle
    divider_weight: HtmlDividerWeight
    divider_color: HtmlDividerColor
    text_style: HtmlTextStyle
    text_align: HtmlTextAlign
    text_color: HtmlTextColor
    background: HtmlBackground


class BaseColumnInput(TypedDict, total=False):
    width: ColumnWidth
    padding: ColumnPadding
    divider: ColumnDivider
    text_style: TextStyle
    text_align: TextAlign
    text_color: TextColor
    background: Background
    html_styles: Optional[HtmlColumnInput]


class GridColumnInput(BaseColumnInput):
    string: TextString


class BaseTableColumnInput(BaseColumnInput, total=False):
    string: TextString


class TableColumnInput(BaseTableColumnInput):
    key: str


TableRowsInput = Dict[str, TextString]


########################################################################################################################
########################################################################################################################
########################################################################################################################


class BaseStylesInput(TypedDict, total=False):
    frame_styles: FrameStylesInput
    column_styles: ColumnStylesInput
    text_styles: TextStylesInput
#
#
# GridStylesInput = BaseStylesInput
#
#
TableHeaderFrameStylesInput = BaseStylesInput


class TableBodyFrameStylesInput(BaseStylesInput, total=False):
    row_styles: RowsStylesInput


########################################################################################################################


class HtmlStylesInput(TypedDict, total=False):
    html_frame_styles: HtmlFrameStylesInput
    html_column_styles: HtmlColumnStylesInput
    html_text_styles: HtmlTextStylesInput


class HtmlTextFrameStylesInput(TypedDict, total=False):
    html_frame_styles: HtmlFrameStylesInput
    html_text_styles: HtmlTextStylesInput


HtmlGridFrameStylesInput = HtmlStylesInput


class HtmlTableFrameStylesInput(HtmlStylesInput, total=False):
    html_row_styles: HtmlRowsStylesInput


HtmlTableHeaderStylesInput = HtmlStylesInput

# HtmlTableBodyStylesInput = HtmlTableStylesInput

########################################################################################################################
########################################################################################################################
########################################################################################################################

